package echo;

import com.sun.grizzly.tcp.Adapter;
import com.sun.grizzly.tcp.InputBuffer;
import com.sun.grizzly.tcp.OutputBuffer;
import com.sun.grizzly.tcp.Request;
import com.sun.grizzly.tcp.Response;
import com.sun.grizzly.util.buf.ByteChunk;

/**
 * Send back asyncronously the payload that he receive.
 */
public class EchoAsyncAdapter implements Adapter
{
	@Override
	public void service( Request request, Response response ) throws Exception
	{
		EchoRunner runner = new EchoRunner( request, response );
		new Thread( runner, "Echo:" + request.requestURI().toString() ).start();
		response.suspend();
	}

	@Override
	public void afterService( Request request, Response response ) throws Exception
	{
		request.recycle();
		response.recycle();
	}

	private class EchoRunner implements Runnable
	{
		private Request request;
		private Response response;

		public EchoRunner( Request request, Response response )
		{
			this.request = request;
			this.response = response;
		}

		@Override
		public void run()
		{
			try
			{
				InputBuffer in = request.getInputBuffer();
				OutputBuffer out = response.getOutputBuffer();
				response.setStatus( 200 );
				ByteChunk chunk = new ByteChunk( 1024 );
				String str = "##Echo_Top\n";
				chunk.append( str.getBytes(), 0, str.length() );
				out.doWrite( chunk, response );
				while (in.doRead( chunk, request ) > 0)
				{
					out.doWrite( chunk, response );
				}
				str = "##Echo_Bottom\n";
				chunk.append( str.getBytes(), 0, str.length() );
				out.doWrite( chunk, response );
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
			}
			finally
			{
				try
				{
					response.flush();
				}
				catch (Exception ex)
				{
					ex.printStackTrace();
				}
				response.resume();
			}
		}
	}
}
